package io.gitlab.jfronny.libjf;

import net.neoforged.fml.ModList;
import net.neoforged.neoforgespi.language.IModInfo;
import org.jetbrains.annotations.ApiStatus;

import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.function.BiFunction;

@ApiStatus.Internal
public class Flags {
    public static Set<StringFlag> getStringFlags(String name) {
        return getFlags(name,
                (source, value) -> new StringFlag(source, value),
                (source, value) -> value instanceof String s ? new StringFlag(source, s) : null);
    }

    public static Set<BooleanFlag> getBoolFlags(String name) {
        return getFlags(name,
                (source, value) -> new BooleanFlag(source, true),
                (source, value) -> value instanceof Boolean b ? new BooleanFlag(source, b) : null);
    }

    private static <T> Set<T> getFlags(String name, BiFunction<String, String, T> makeFromProp, BiFunction<String, Object, T> makeFromFmj) {
        Set<T> flags = new LinkedHashSet<>();
        String propName = "libjf." + name;
        if (System.getProperty(propName) != null) {
            T created = makeFromProp.apply("System Property", System.getProperty(propName));
            if (created != null) flags.add(created);
        }
        for (IModInfo mod : ModList.get().getMods()) {
            Map<String, Object> co = (Map<String, Object>) mod.getModProperties().get("libjf");
            if (co == null || !co.containsKey(name)) continue;
            T created = makeFromFmj.apply("Mod: " + mod.getModId(), co.get(name));
            if (created != null) flags.add(created);
        }
        return flags;
    }

    public record StringFlag(String source, String value) {
    }

    public record BooleanFlag(String source, boolean value) {
    }
}
