package io.gitlab.jfronny.libjf.coprocess;

import io.gitlab.jfronny.libjf.LibJf;
import net.neoforged.api.distmarker.Dist;
import net.neoforged.bus.api.SubscribeEvent;
import net.neoforged.fml.event.lifecycle.FMLCommonSetupEvent;
import net.neoforged.fml.loading.FMLEnvironment;
import net.neoforged.neoforge.event.server.ServerStoppedEvent;

import java.io.Closeable;
import java.io.IOException;
import java.util.*;

public class CoProcessManager {
    private boolean started = false;
    private final List<CoProcess> coProcesses = new ArrayList<>();

    @SubscribeEvent
    public void onInitialize(FMLCommonSetupEvent event) {
        synchronized (coProcesses) {
            started = true;
        }
        Runtime.getRuntime().addShutdownHook(new Thread(this::stop));
        start();
    }

    @SubscribeEvent
    public void onServerStop(ServerStoppedEvent event) {
        if (FMLEnvironment.dist == Dist.DEDICATED_SERVER) stop();
    }

    public void register(CoProcess coProcess) {
        synchronized (coProcesses) {
            if (started) throw new IllegalStateException("Cannot register co-process after initialization");
            coProcesses.add(coProcess);
        }
    }

    private void start() {
        for (CoProcess coProcess : coProcesses) {
            coProcess.start();
        }
    }

    private void stop() {
        for (Iterator<CoProcess> iter = coProcesses.iterator(); iter.hasNext(); ) {
            CoProcess coProcess = iter.next();
            coProcess.stop();
            if (coProcess instanceof Closeable cl) {
                try {
                    cl.close();
                } catch (IOException e) {
                    LibJf.LOGGER.error("Could not close co-process", e);
                }
            }
            iter.remove();
        }
    }
}
